@@ -19,7 +19,7 @@ Test::Mockify - minimal mocking framework for perl
   my $App = SampleApp->new('logger'=> $MockedLogger);
   $App->do_something();
 
-  # verify that the mock object was called
+  # verify that the mocked method was called
   ok(WasCalled($MockedLogger, 'log'), 'log was called');
   done_testing();
 
@@ -37,6 +37,7 @@ use Test::Mockify::Tools qw ( Error ExistsMethod IsValid LoadPackage Isa );
 use Test::Mockify::TypeTests qw ( IsInteger IsFloat IsString IsArrayReference IsHashReference IsObjectReference );
 use Test::Mockify::MethodCallCounter;
 use Test::Mockify::Method;
+use Test::Mockify::MethodSpy;
 use Test::MockObject::Extends;
 use Test::Mockify::CompatibilityTools qw (MigrateOldMatchers);
 use Data::Dumper;
@@ -47,7 +48,7 @@ use experimental 'switch';
 
 use strict;
 
-our $VERSION = '0.10.0';
+our $VERSION = '0.10.1';
 
 sub new {
     my $class = shift;
@@ -68,15 +69,15 @@ sub new {
 sub _mockedModulPath {
     my $self = shift;
     my ($ModulPath) = @_;
-    return $self->{'__MockedModulePath'} unless ($ModulPath);
-    $self->{'__MockedModulePath'} = $ModulPath;
+    return $self->{'MockedModulePath'} unless ($ModulPath);
+    $self->{'MockedModulePath'} = $ModulPath;
 }
 #----------------------------------------------------------------------------------------
 sub _mockedSelf {
     my $self = shift;
     my ($MockedSelf) = @_;
-    return $self->{'__MockedModule'} unless ($MockedSelf);
-    $self->{'__MockedModule'} = $MockedSelf;
+    return $self->{'MockedModule'} unless ($MockedSelf);
+    $self->{'MockedModule'} = $MockedSelf;
 }
 #----------------------------------------------------------------------------------------
 sub _initMockedModule {
@@ -128,11 +129,11 @@ For example, the next line will create a mocked version of the method log, but o
 
 =head4 when
 
-To define the signatur in the needed structure you should use the L<< Test::Mockify::Matchers >>.
+To define the signature in the needed structure you must use the L<< Test::Mockify::Matcher >>.
 
 =head4 whenAny
 
-If you don't want to specify the method signatur at all, you can use whenAny.
+If you don't want to specify the method signature at all, you can use whenAny.
 It is not possible to mix C<whenAny> and C<when> for the same method.
 
 =head4 then ...
@@ -143,9 +144,10 @@ For possible return types please look in L<Test::Mockify::ReturnValue>
 sub mock {
     my $self = shift;
     my @Parameters = @_;
+    
     my $ParameterAmount = scalar @Parameters;
     if($ParameterAmount == 1 && IsString($Parameters[0]) ){
-        return $self->_addMethod($Parameters[0]);
+        return $self->_addMockWithMethod($Parameters[0]);
     }
     if($ParameterAmount == 2){
         my ( $MethodName, $ReturnValueOrFunctionPointer ) = @Parameters;
@@ -161,10 +163,53 @@ sub mock {
     }
     return;
 }
+=pod
+
+=head2 spy
+
+Use spy if you want to observe a method. You can use the L<Test::Mockify::Verify> to ensure that the method was called with the expected parameters.
+
+=head3 synopsis
+
+This method takes one parameter, which is the name of the method you like to spy.
+Because you need to specify more detailed the behaviour of this spy you have to define the method signature with C<when>
+
+For example, the next line will create a method spy of the method log, but only if this method is called with any string and the number 123. Mockify will throw an error if this method is called in another way.
+
+  my $MockObjectBuilder = Test::Mockify->new( 'Sample::Logger', [] );
+  $MockObjectBuilder->spy('log')->when(String(), Number(123));
+  my $SampleLogger = $MockObjectBuilder->getMockObject();
+
+  # call spied method
+  $SampleLogger->log('abc', 123);
+
+  # verify that the spied method was called
+  is_deeply(GetParametersFromMockifyCall($MockedLogger, 'log'),['abc', 123], 'Check parameters of first call');
+
+=head4 when
+
+To define the signature in the needed structure you must use the L<< Test::Mockify::Matcher >>.
+
+=head4 whenAny
+
+If you don't want to specify the method signature at all, you can use whenAny.
+It is not possible to mix C<whenAny> and C<when> for the same method.
+
+=cut
+sub spy {
+    my $self = shift;
+    my ($MethodName) = @_;
+    my $PointerOriginalMethod = \&{$self->_mockedModulPath().'::'.$MethodName};
+    #In order to have the current object available in the parameter list, it has to be injected here.
+    return $self->_addMockWithMethodSpy($MethodName, sub {
+        return $PointerOriginalMethod->($self->_mockedSelf(), @_);
+    });
+}
+
 #----------------------------------------------------------------------------------------
 =pod
 
-=head2 addMethodSpy
+=head2 addMethodSpy I<(deprecated)>
 
 With this method it is possible to observe a method. That means, you keep the original functionality but you can get meta data from the mockify-framework.
 
@@ -174,36 +219,36 @@ With this method it is possible to observe a method. That means, you keep the or
 sub addMethodSpy {
     my $self = shift;
     my ( $MethodName ) = @_;
-    my $PointerOriginalMethod = \&{$self->_mockedModulPath().'::'.$MethodName};
-    $self->_addMethod($MethodName)->whenAny()->thenCall(sub {
-        return $PointerOriginalMethod->($self->_mockedSelf(), @_);
-    });
+    if (warnings::enabled("deprecated")) {
+        warnings::warn('deprecated', "addMethodSpy is deprecated, use spy('name')->whenAny()");
+    }
+    $self->spy($MethodName)->whenAny();
     return;
 }
 #----------------------------------------------------------------------------------------
 =pod
 
-=head2 addMethodSpyWithParameterCheck
+=head2 addMethodSpyWithParameterCheck I<(deprecated)>
 
 With this method it is possible to observe a method and check the parameters. That means, you keep the original functionality, but you can get meta data from the mockify- framework and use the parameter check, like B<addMockWithReturnValueAndParameterCheck>.
 
   my $aParameterTypes = [String(),String(abcd)];
   $MockObjectBuilder->addMethodSpyWithParameterCheck('myMethodName', $aParameterTypes);
 
-To define in a nice way the signatur you should use the L<< Test::Mockify::Matchers; >>.
+To define it in a nice way the signature you must use the L<< Test::Mockify::Matcher; >>.
 
 =cut
 sub addMethodSpyWithParameterCheck {
     my $self = shift;
     my ( $MethodName, $aParameterTypes ) = @_;
-
-    my $PointerOriginalMethod = \&{$self->_mockedModulPath().'::'.$MethodName};
-    $aParameterTypes = MigrateOldMatchers($aParameterTypes);
-    $self->_addMethod($MethodName)->when(@{$aParameterTypes})->thenCall(sub {
-        return $PointerOriginalMethod->($self->_mockedSelf(), @_);
-    });
+    if (warnings::enabled("deprecated")) {
+        warnings::warn('deprecated', "addMethodSpyWithParameterCheck is deprecated, use spy('name')->when(String('abc'))");
+    }
+    my $aMigratedMatchers = MigrateOldMatchers($aParameterTypes);
+    $self->spy($MethodName)->when(@{$aMigratedMatchers});
     return;
 }
+
 #----------------------------------------------------------------------------------------
 =pod
 
@@ -225,21 +270,35 @@ sub addMock {
     if (warnings::enabled("deprecated")) {
         warnings::warn('deprecated', "addMock is deprecated, use mock('name')->whenAny()->thenCall(sub{})");
     }
-    $self->_addMethod($MethodName)->whenAny()->thenCall(sub {
+    $self->_addMockWithMethod($MethodName)->whenAny()->thenCall(sub {
         return $rSub->($self->_mockedSelf(), @_);
     });
 
     return;
 }
 #-------------------------------------------------------------------------------------
-sub _addMethod {
+sub _addMockWithMethod {
     my $self = shift;
     my ( $MethodName ) = @_;
+    $self->_testMockTypeUsage($MethodName);
+    return $self->_addMock($MethodName, Test::Mockify::Method->new());
+}
+#-------------------------------------------------------------------------------------
+sub _addMockWithMethodSpy {
+    my $self = shift;
+    my ( $MethodName, $PointerOriginalMethod ) = @_;
+    $self->_testMockTypeUsage($MethodName);
+    return $self->_addMock($MethodName, Test::Mockify::MethodSpy->new($PointerOriginalMethod));
+}
+#-------------------------------------------------------------------------------------
+sub _addMock {
+    my $self = shift;
+    my ( $MethodName, $Method) = @_;
 
     ExistsMethod( $self->_mockedModulPath(), $MethodName );
     $self->_mockedSelf()->{'__MethodCallCounter'}->addMethod( $MethodName );
     if(not $self->{'MethodStore'}{$MethodName}){
-        $self->{'MethodStore'}{$MethodName} //= Test::Mockify::Method->new();
+        $self->{'MethodStore'}{$MethodName} //= $Method;
         $self->_mockedSelf()->mock($MethodName, sub {
             $self->_mockedSelf()->{'__MethodCallCounter'}->increment( $MethodName );
             my $MockedSelf = shift;
@@ -267,9 +326,9 @@ sub addMockWithReturnValue {
         warnings::warn('deprecated', "addMockWithReturnValue is deprecated, use mock('name')->when()->thenReturn('Value')");
     }
     if($ReturnValue){
-        $self->_addMethod($MethodName)->when()->thenReturn($ReturnValue);
+        $self->_addMockWithMethod($MethodName)->when()->thenReturn($ReturnValue);
     }else {
-        $self->_addMethod($MethodName)->when()->thenReturnUndef();
+        $self->_addMockWithMethod($MethodName)->when()->thenReturnUndef();
     }
 
     return;
@@ -288,7 +347,7 @@ In the following example two strings will be expected, and the second one has to
   my $aParameterTypes = [String(),String('abcd')];
   $MockObjectBuilder->addMockWithReturnValueAndParameterCheck('myMethodName','the return value',$aParameterTypes);
 
-To define in a nice way the signatur you should use the L<< Test::Mockify::Matchers; >>.
+To define it in a nice way the signature you must use the L<< Test::Mockify::Matcher; >>.
 
 =cut
 sub addMockWithReturnValueAndParameterCheck {
@@ -306,9 +365,9 @@ sub addMockWithReturnValueAndParameterCheck {
     $aParameterTypes = MigrateOldMatchers($aParameterTypes);
 
     if($ReturnValue){
-        $self->_addMethod($MethodName)->when(@{$aParameterTypes})->thenReturn($ReturnValue);
+        $self->_addMockWithMethod($MethodName)->when(@{$aParameterTypes})->thenReturn($ReturnValue);
     }else {
-        $self->_addMethod($MethodName)->when(@{$aParameterTypes})->thenReturnUndef();
+        $self->_addMockWithMethod($MethodName)->when(@{$aParameterTypes})->thenReturnUndef();
     }
 
     return;
@@ -352,7 +411,19 @@ sub _addGetParameterFromMockifyCall {
 
     return;
 }
-
+#----------------------------------------------------------------------------------------
+sub _testMockTypeUsage {
+    my $self = shift;
+    my ($MethodName) = @_;
+    my $PositionInCallerStack = 2;
+    my $MethodMockType = (caller($PositionInCallerStack))[3]; # autodetect mock type (spy or mock)
+    if($self->{'MethodMockType'}{$MethodName} && $self->{'MethodMockType'}{$MethodName} ne $MethodMockType){
+        die('It is not possible to mix spy and mock');
+    }else{
+        $self->{'MethodMockType'}{$MethodName} = $MethodMockType;
+    }
+    return;
+}
 1;
 
 __END__
